@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,7 +5,7 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /*
@@ -17,35 +14,27 @@
  *  This library forms the socket for run-time loadable render plugins.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
+#include <assert.h>
 #include <string.h>
-#include "memory.h"
-#include "const.h"
-#include "macros.h"
-#include "colorprocs.h"
-#include "gvplugin_render.h"
-#include "cgraph.h"
-#include "gvcint.h"
-#include "geom.h"
-#include "geomprocs.h"
-#include "gvcproc.h"
+#include <common/memory.h>
+#include <common/const.h>
+#include <common/macros.h>
+#include <common/colorprocs.h>
+#include <gvc/gvplugin_render.h>
+#include <cgraph/cgraph.h>
+#include <gvc/gvcint.h>
+#include <common/geom.h>
+#include <common/geomprocs.h>
+#include <gvc/gvcproc.h>
+#include <cgraph/strcasecmp.h>
+#include <stdlib.h>
 
 extern int emit_once(char *str);
 extern shape_desc *find_user_shape(char *name);
 extern boolean mapbool(char *s);
 
-#ifndef HAVE_STRCASECMP
-extern int strcasecmp(const char *s1, const char *s2);
-#endif
-
-/* storage for temporary hacks until client API is FP */
-static pointf *AF;
-static int sizeAF;
-/* end hack */
-
 int gvrender_select(GVJ_t * job, const char *str)
 {
     GVC_t *gvc = job->gvc;
@@ -187,9 +176,16 @@ pointf *gvrender_ptf_A(GVJ_t * job, pointf * af, pointf * AF, int n)
 
 static int gvrender_comparestr(const void *s1, const void *s2)
 {
-    return strcmp(*(char **) s1, *(char **) s2);
+    return strcmp(*(char *const *) s1, *(char *const *) s2);
 }
 
+/* gvrender_resolve_color:
+ * N.B. strcmp cannot be used in bsearch, as it will pass a pointer
+ * to an element in the array features->knowncolors (i.e., a char**)
+ * as an argument of the compare function, while the arguments to 
+ * strcmp are both char*. Given this, the first argument to
+ * bsearch must also be char**, so we use &tok.
+ */
 static void gvrender_resolve_color(gvrender_features_t * features,
 				   char *name, gvcolor_t * color)
 {
@@ -462,7 +458,7 @@ void gvrender_set_pencolor(GVJ_t * job, char *name)
     gvcolor_t *color = &(job->obj->pencolor);
     char *cp = NULL;
 
-    if ((cp = strstr(name, ":")))	/* if its a color list, then use only first */
+    if ((cp = strchr(name, ':'))) // if it’s a color list, then use only first
 	*cp = '\0';
     if (gvre) {
 	gvrender_resolve_color(job->render.features, name, color);
@@ -479,7 +475,7 @@ void gvrender_set_fillcolor(GVJ_t * job, char *name)
     gvcolor_t *color = &(job->obj->fillcolor);
     char *cp = NULL;
 
-    if ((cp = strstr(name, ":")))	/* if its a color list, then use only first */
+    if ((cp = strchr(name, ':'))) // if it’s a color list, then use only first
 	*cp = '\0';
     if (gvre) {
 	gvrender_resolve_color(job->render.features, name, color);
@@ -581,12 +577,12 @@ void gvrender_polygon(GVJ_t * job, pointf * af, int n, int filled)
 	    if (job->flags & GVRENDER_DOES_TRANSFORM)
 		gvre->polygon(job, af, n, filled);
 	    else {
-		if (sizeAF < n) {
-		    sizeAF = n + 10;
-		    AF = grealloc(AF, sizeAF * sizeof(pointf));
-		}
+		pointf *AF;
+		assert(n >= 0);
+		AF = gcalloc((size_t)n, sizeof(pointf));
 		gvrender_ptf_A(job, af, AF, n);
 		gvre->polygon(job, AF, n, filled);
+		free(AF);
 	    }
 	    if (noPoly)
 		job->obj->pencolor = save_pencolor;
@@ -610,8 +606,7 @@ void gvrender_box(GVJ_t * job, boxf B, int filled)
 }
 
 void gvrender_beziercurve(GVJ_t * job, pointf * af, int n,
-			  int arrow_at_start, int arrow_at_end,
-			  boolean filled)
+			  int arrow_at_start, int arrow_at_end, int filled)
 {
     gvrender_engine_t *gvre = job->render.engine;
 
@@ -621,13 +616,13 @@ void gvrender_beziercurve(GVJ_t * job, pointf * af, int n,
 		gvre->beziercurve(job, af, n, arrow_at_start, arrow_at_end,
 				  filled);
 	    else {
-		if (sizeAF < n) {
-		    sizeAF = n + 10;
-		    AF = grealloc(AF, sizeAF * sizeof(pointf));
-		}
+		pointf *AF;
+		assert(n >= 0);
+		AF = gcalloc((size_t)n, sizeof(pointf));
 		gvrender_ptf_A(job, af, AF, n);
 		gvre->beziercurve(job, AF, n, arrow_at_start, arrow_at_end,
 				  filled);
+		free(AF);
 	    }
 	}
     }
@@ -642,12 +637,12 @@ void gvrender_polyline(GVJ_t * job, pointf * af, int n)
 	    if (job->flags & GVRENDER_DOES_TRANSFORM)
 		gvre->polyline(job, af, n);
 	    else {
-		if (sizeAF < n) {
-		    sizeAF = n + 10;
-		    AF = grealloc(AF, sizeAF * sizeof(pointf));
-		}
+		pointf *AF;
+		assert(n >= 0);
+		AF = gcalloc((size_t)n, sizeof(pointf));
 		gvrender_ptf_A(job, af, AF, n);
 		gvre->polyline(job, AF, n);
+		free(AF);
 	    }
 	}
     }
@@ -681,11 +676,37 @@ static imagescale_t get_imagescale(char *s)
     return IMAGESCALE_FALSE;
 }
 
+static imagepos_t get_imagepos(char *s)
+{
+    if (*s == '\0')
+	return IMAGEPOS_MIDDLE_CENTER;
+    if (!strcasecmp(s, "tl"))
+	return IMAGEPOS_TOP_LEFT;
+    if (!strcasecmp(s, "tc"))
+	return IMAGEPOS_TOP_CENTER;
+    if (!strcasecmp(s, "tr"))
+	return IMAGEPOS_TOP_RIGHT;
+    if (!strcasecmp(s, "ml"))
+	return IMAGEPOS_MIDDLE_LEFT;
+    if (!strcasecmp(s, "mc"))
+	return IMAGEPOS_MIDDLE_CENTER;
+    if (!strcasecmp(s, "mr"))
+	return IMAGEPOS_MIDDLE_RIGHT;
+    if (!strcasecmp(s, "bl"))
+	return IMAGEPOS_BOTTOM_LEFT;
+    if (!strcasecmp(s, "bc"))
+	return IMAGEPOS_BOTTOM_CENTER;
+    if (!strcasecmp(s, "br"))
+	return IMAGEPOS_BOTTOM_RIGHT;
+    return IMAGEPOS_MIDDLE_CENTER;
+}
+
 /* gvrender_usershape:
- * Scale image to fill polygon bounding box according to "imagescale"
+ * Scale image to fill polygon bounding box according to "imagescale",
+ * positioned at "imagepos"
  */
 void gvrender_usershape(GVJ_t * job, char *name, pointf * a, int n,
-			boolean filled, char *imagescale)
+			boolean filled, char *imagescale, char *imagepos)
 {
     gvrender_engine_t *gvre = job->render.engine;
     usershape_t *us;
@@ -694,6 +715,7 @@ void gvrender_usershape(GVJ_t * job, char *name, pointf * a, int n,
     boxf b;			/* target box */
     int i;
     point isz;
+    imagepos_t position;
 
     assert(job);
     assert(name);
@@ -751,14 +773,45 @@ void gvrender_usershape(GVJ_t * job, char *name, pointf * a, int n,
 	break;
     }
 
-    /* if image is smaller than target area then center it */
+    /* if image is smaller in any dimension, apply the specified positioning */
+    position = get_imagepos(imagepos);
     if (iw < pw) {
-	b.LL.x += (pw - iw) / 2.0;
-	b.UR.x -= (pw - iw) / 2.0;
+        switch (position) {
+        case IMAGEPOS_TOP_LEFT:
+        case IMAGEPOS_MIDDLE_LEFT:
+        case IMAGEPOS_BOTTOM_LEFT:
+            b.UR.x = b.LL.x + iw;
+            break;
+        case IMAGEPOS_TOP_RIGHT:
+        case IMAGEPOS_MIDDLE_RIGHT:
+        case IMAGEPOS_BOTTOM_RIGHT:
+            b.LL.x += (pw - iw);
+            b.UR.x = b.LL.x + iw;
+            break;
+        default:
+            b.LL.x += (pw - iw) / 2.0;
+            b.UR.x -= (pw - iw) / 2.0;
+            break;
+        }
     }
     if (ih < ph) {
-	b.LL.y += (ph - ih) / 2.0;
-	b.UR.y -= (ph - ih) / 2.0;
+        switch (position) {
+        case IMAGEPOS_TOP_LEFT:
+        case IMAGEPOS_TOP_CENTER:
+        case IMAGEPOS_TOP_RIGHT:
+            b.LL.y = b.UR.y - ih;
+            break;
+        case IMAGEPOS_BOTTOM_LEFT:
+        case IMAGEPOS_BOTTOM_CENTER:
+        case IMAGEPOS_BOTTOM_RIGHT:
+            b.LL.y += ih;
+            b.UR.y = b.LL.y - ih;
+            break;
+        default:
+            b.LL.y += (ph - ih) / 2.0;
+            b.UR.y -= (ph - ih) / 2.0;
+            break;
+        }
     }
 
     /* convert from graph to device coordinates */
@@ -788,6 +841,5 @@ void gvrender_set_penwidth(GVJ_t * job, double penwidth)
 
     if (gvre) {
 	job->obj->penwidth = penwidth;
-	/*if (gvre->set_penwidth) gvre->set_penwidth(job, penwidth); */
     }
 }
