@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,45 +5,45 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include "cgraph.h"     /* for agerr() and friends */
-#include "delaunay.h"
-#include "memory.h"
-#include "logic.h"
+#include <cgraph/cgraph.h>     /* for agerr() and friends */
+#include <neatogen/delaunay.h>
+#include <common/memory.h>
+#include <common/logic.h>
 
-#if HAVE_GTS
+#ifdef HAVE_GTS
 #include <gts.h>
 
-static gboolean triangle_is_hole(GtsTriangle * t)
+static gint triangle_is_hole(void *triangle, void *ignored)
 {
+    GtsTriangle *t = triangle;
+    (void)ignored;
+
     GtsEdge *e1, *e2, *e3;
     GtsVertex *v1, *v2, *v3;
-    gboolean ret;
 
     gts_triangle_vertices_edges(t, NULL, &v1, &v2, &v3, &e1, &e2, &e3);
 
     if ((GTS_IS_CONSTRAINT(e1) && GTS_SEGMENT(e1)->v1 != v1) ||
 	(GTS_IS_CONSTRAINT(e2) && GTS_SEGMENT(e2)->v1 != v2) ||
 	(GTS_IS_CONSTRAINT(e3) && GTS_SEGMENT(e3)->v1 != v3))
-	ret = TRUE;
-    else ret = FALSE;
-    return ret;
+	return TRUE;
+
+    return FALSE;
 }
 
 static guint delaunay_remove_holes(GtsSurface * surface)
 {
-    return gts_surface_foreach_face_remove(surface,
-				    (GtsFunc) triangle_is_hole, NULL);
+    return gts_surface_foreach_face_remove(surface, triangle_is_hole, NULL);
 }
 
 /* Derived classes for vertices and faces so we can assign integer ids
@@ -177,8 +174,8 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
      */
     for (i = 0; i < nsegs; i++) {
 	edges[i] = gts_edge_new(ecl,
-		 (GtsVertex *) (vertices[ segs[ 2 * i]]),
-		 (GtsVertex *) (vertices[ segs[ 2 * i + 1]]));
+		 (GtsVertex *)vertices[segs[2 * i]],
+		 (GtsVertex *)vertices[segs[2 * i + 1]]);
     }
 
     for (i = 0; i < npt; i++)
@@ -196,15 +193,14 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 					       t->e1, t->e2, t->e3));
 
     for (i = 0; i < npt; i++) {
-	GtsVertex *v1 = (GtsVertex *) vertices[i];
-	GtsVertex *v = gts_delaunay_add_vertex(surface, v1, NULL);
+	GtsVertex *v4 = (GtsVertex *)vertices[i];
+	GtsVertex *v = gts_delaunay_add_vertex(surface, v4, NULL);
 
 	/* if v != NULL, it is a previously added pt with the same
-	 * coordinates as v1, in which case we replace v1 with v
+	 * coordinates as v4, in which case we replace v4 with v
 	 */
 	if (v) {
-	    /* agerr (AGWARN, "Duplicate point %d %d\n", i, ((GVertex*)v)->idx); */
-	    gts_vertex_replace (v1, v);
+	    gts_vertex_replace(v4, v);
 	}
     }
 
@@ -215,11 +211,6 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
     /* destroy enclosing triangle */
     gts_allow_floating_vertices = TRUE;
     gts_allow_floating_edges = TRUE;
-/*
-    gts_object_destroy(GTS_OBJECT(v1));
-    gts_object_destroy(GTS_OBJECT(v2));
-    gts_object_destroy(GTS_OBJECT(v3));
-*/
     destroy(v1);
     destroy(v2);
     destroy(v3);
@@ -239,32 +230,39 @@ typedef struct {
     v_data *delaunay;
 } estats;
     
-static void cnt_edge (GtsSegment * e, estats* sp)
-{
+static gint cnt_edge(void *edge, void *stats) {
+    GtsSegment *e = edge;
+    estats *sp = stats;
+
     sp->n++;
     if (sp->delaunay) {
-	sp->delaunay[((GVertex*)(e->v1))->idx].nedges++;
-	sp->delaunay[((GVertex*)(e->v2))->idx].nedges++;
+	sp->delaunay[((GVertex*)e->v1)->idx].nedges++;
+	sp->delaunay[((GVertex*)e->v2)->idx].nedges++;
     }
+
+    return 0;
 }
 
 static void
 edgeStats (GtsSurface* s, estats* sp)
 {
-    gts_surface_foreach_edge (s, (GtsFunc) cnt_edge, sp);
+    gts_surface_foreach_edge(s, cnt_edge, sp);
 }
 
-static void add_edge (GtsSegment * e, v_data* delaunay)
-{
-    int source = ((GVertex*)(e->v1))->idx;
-    int dest = ((GVertex*)(e->v2))->idx;
+static gint add_edge(void *edge, void *data) {
+    GtsSegment *e = edge;
+    v_data *delaunay = data;
+
+    int source = ((GVertex*)e->v1)->idx;
+    int dest = ((GVertex*)e->v2)->idx;
 
     delaunay[source].edges[delaunay[source].nedges++] = dest;
     delaunay[dest].edges[delaunay[dest].nedges++] = source;
+
+    return 0;
 }
 
-v_data *delaunay_triangulation(double *x, double *y, int n)
-{
+static v_data *delaunay_triangulation(double *x, double *y, int n) {
     v_data *delaunay;
     GtsSurface* s = tri(x, y, n, NULL, 0, 1);
     int i, nedges;
@@ -292,7 +290,7 @@ v_data *delaunay_triangulation(double *x, double *y, int n)
 	delaunay[i].edges[0] = i;
 	delaunay[i].nedges = 1;
     }
-    gts_surface_foreach_edge (s, (GtsFunc) add_edge, delaunay);
+    gts_surface_foreach_edge(s, add_edge, delaunay);
 
     gts_object_destroy (GTS_OBJECT (s));
 
@@ -304,19 +302,22 @@ typedef struct {
     int* edges;
 } estate;
 
-static void addEdge (GtsSegment * e, estate* es)
-{
-    int source = ((GVertex*)(e->v1))->idx;
-    int dest = ((GVertex*)(e->v2))->idx;
+static gint addEdge(void *edge, void *state) {
+    GtsSegment *e = edge;
+    estate *es = state;
 
-    es->edges[2*(es->n)] = source;
-    es->edges[2*(es->n)+1] = dest;
+    int source = ((GVertex*)e->v1)->idx;
+    int dest = ((GVertex*)e->v2)->idx;
+
+    es->edges[2 * es->n] = source;
+    es->edges[2 * es->n + 1] = dest;
     es->n += 1;
+
+    return 0;
 }
 
-/* If qsort_r ever becomes standardized, this should be used
- * instead of having a global variable.
- */
+// when moving to C11, qsort_s should be used instead of having a global
+// variable
 static double* _vals;
 typedef int (*qsort_cmpf) (const void *, const void *);
 
@@ -362,7 +363,7 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
 	edges = N_GNEW(2 * nedges, int);
 	state.n = 0;
 	state.edges = edges;
-	gts_surface_foreach_edge (s, (GtsFunc) addEdge, &state);
+	gts_surface_foreach_edge(s, addEdge, &state);
     }
     else {
 	int* vs = N_GNEW(n, int);
@@ -397,10 +398,14 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
     return edges;
 }
 
-static void cntFace (GFace* fp, int* ip)
-{
+static gint cntFace(void *face, void *data) {
+    GFace *fp = face;
+    int *ip = data;
+
     fp->idx = *ip;
     *ip += 1;
+
+    return 0;
 }
 
 typedef struct {
@@ -414,14 +419,20 @@ typedef struct {
     int* neigh;
 } ninfo;
 
-static void addNeighbor (GFace* f, ninfo* es)
-{
+static gint addNeighbor(void *face, void *ni) {
+    GFace *f = face;
+    ninfo *es = ni;
+
     es->neigh[es->nneigh] = f->idx;
     es->nneigh++;
+
+    return 0;
 }
 
-static void addFace (GFace* f, fstate* es)
-{
+static gint addFace(void *face, void *state) {
+    GFace *f = face;
+    fstate *es = state;
+
     int i, myid = f->idx;
     int* ip = es->faces + 3*myid;
     int* neigh = es->neigh + 3*myid;
@@ -429,27 +440,33 @@ static void addFace (GFace* f, fstate* es)
     GtsVertex *v1, *v2, *v3;
 
     gts_triangle_vertices (&f->v.triangle, &v1, &v2, &v3);
-    *ip++ = ((GVertex*)(v1))->idx;
-    *ip++ = ((GVertex*)(v2))->idx;
-    *ip++ = ((GVertex*)(v3))->idx;
+    *ip++ = ((GVertex*)v1)->idx;
+    *ip++ = ((GVertex*)v2)->idx;
+    *ip++ = ((GVertex*)v3)->idx;
 
     ni.nneigh = 0;
     ni.neigh = neigh;
-    gts_face_foreach_neighbor ((GtsFace*)f, 0, (GtsFunc) addNeighbor, &ni);
+    gts_face_foreach_neighbor((GtsFace*)f, 0, addNeighbor, &ni);
     for (i = ni.nneigh; i < 3; i++)
 	neigh[i] = -1;
+
+    return 0;
 }
 
-static void addTri (GFace* f, fstate* es)
-{
+static gint addTri(void *face, void *state) {
+    GFace *f = face;
+    fstate *es = state;
+
     int myid = f->idx;
     int* ip = es->faces + 3*myid;
     GtsVertex *v1, *v2, *v3;
 
     gts_triangle_vertices (&f->v.triangle, &v1, &v2, &v3);
-    *ip++ = ((GVertex*)(v1))->idx;
-    *ip++ = ((GVertex*)(v2))->idx;
-    *ip++ = ((GVertex*)(v3))->idx;
+    *ip++ = ((GVertex*)v1)->idx;
+    *ip++ = ((GVertex*)v2)->idx;
+    *ip++ = ((GVertex*)v3)->idx;
+
+    return 0;
 }
 
 /* mkSurface:
@@ -482,16 +499,16 @@ mkSurface (double *x, double *y, int n, int* segs, int nsegs)
 
     state.n = 0;
     state.edges = segs;
-    gts_surface_foreach_edge (s, (GtsFunc) addEdge, &state);
+    gts_surface_foreach_edge(s, addEdge, &state);
 
-    gts_surface_foreach_face (s, (GtsFunc) cntFace, &nfaces);
+    gts_surface_foreach_face(s, cntFace, &nfaces);
 
     faces = N_GNEW(3 * nfaces, int);
     neigh = N_GNEW(3 * nfaces, int);
 
     statf.faces = faces;
     statf.neigh = neigh;
-    gts_surface_foreach_face (s, (GtsFunc) addFace, &statf);
+    gts_surface_foreach_face(s, addFace, &statf);
 
     sf->nedges = nsegs;
     sf->edges = segs;
@@ -523,9 +540,9 @@ get_triangles (double *x, int n, int* tris)
     s = tri(x, NULL, n, NULL, 0, 0);
     if (!s) return NULL;
 
-    gts_surface_foreach_face (s, (GtsFunc) cntFace, &nfaces);
+    gts_surface_foreach_face(s, cntFace, &nfaces);
     statf.faces = N_GNEW(3 * nfaces, int);
-    gts_surface_foreach_face (s, (GtsFunc) addTri, &statf);
+    gts_surface_foreach_face(s, addTri, &statf);
 
     gts_object_destroy (GTS_OBJECT (s));
 
@@ -542,9 +559,9 @@ freeSurface (surface_t* s)
 }
 #elif HAVE_TRIANGLE
 #define TRILIBRARY
-#include "triangle.c"
-#include "assert.h"
-#include "general.h"
+#include <triangle.c>
+#include <assert.h>
+#include <sparse/general.h>
 
 int*
 get_triangles (double *x, int n, int* tris)
@@ -556,7 +573,7 @@ get_triangles (double *x, int n, int* tris)
 
     in.numberofpoints = n;
     in.numberofpointattributes = 0;
-    in.pointlist = (REAL *) N_GNEW(in.numberofpoints * 2, REAL);
+    in.pointlist = N_GNEW(in.numberofpoints * 2, REAL);
 
     for (i = 0; i < n; i++){
 	in.pointlist[i*2] = x[i*2];
@@ -568,20 +585,20 @@ get_triangles (double *x, int n, int* tris)
     in.numberofholes = 0;
     in.numberofregions = 0;
     in.regionlist = NULL;
-    mid.pointlist = (REAL *) NULL;            /* Not needed if -N switch used. */
-    mid.pointattributelist = (REAL *) NULL;
-    mid.pointmarkerlist = (int *) NULL; /* Not needed if -N or -B switch used. */
-    mid.trianglelist = (int *) NULL;          /* Not needed if -E switch used. */
-    mid.triangleattributelist = (REAL *) NULL;
-    mid.neighborlist = (int *) NULL;         /* Needed only if -n switch used. */
-    mid.segmentlist = (int *) NULL;
-    mid.segmentmarkerlist = (int *) NULL;
-    mid.edgelist = (int *) NULL;             /* Needed only if -e switch used. */
-    mid.edgemarkerlist = (int *) NULL;   /* Needed if -e used and -B not used. */
-    vorout.pointlist = (REAL *) NULL;        /* Needed only if -v switch used. */
-    vorout.pointattributelist = (REAL *) NULL;
-    vorout.edgelist = (int *) NULL;          /* Needed only if -v switch used. */
-    vorout.normlist = (REAL *) NULL;         /* Needed only if -v switch used. */
+    mid.pointlist = NULL; // Not needed if -N switch used.
+    mid.pointattributelist = NULL;
+    mid.pointmarkerlist = NULL; /* Not needed if -N or -B switch used. */
+    mid.trianglelist = NULL; // Not needed if -E switch used.
+    mid.triangleattributelist = NULL;
+    mid.neighborlist = NULL; // Needed only if -n switch used.
+    mid.segmentlist = NULL;
+    mid.segmentmarkerlist = NULL;
+    mid.edgelist = NULL; // Needed only if -e switch used.
+    mid.edgemarkerlist = NULL; // Needed if -e used and -B not used.
+    vorout.pointlist = NULL; // Needed only if -v switch used.
+    vorout.pointattributelist = NULL;
+    vorout.edgelist = NULL; // Needed only if -v switch used.
+    vorout.normlist = NULL; // Needed only if -v switch used.
 
     /* Triangulate the points.  Switches are chosen to read and write a  */
     /*   PSLG (p), preserve the convex hull (c), number everything from  */
@@ -594,28 +611,28 @@ get_triangles (double *x, int n, int* tris)
 
     *tris = mid.numberoftriangles;
     
-    FREE(in.pointlist);
-    FREE(in.pointattributelist);
-    FREE(in.pointmarkerlist);
-    FREE(in.regionlist);
-    FREE(mid.pointlist);
-    FREE(mid.pointattributelist);
-    FREE(mid.pointmarkerlist);
-    FREE(mid.triangleattributelist);
-    FREE(mid.neighborlist);
-    FREE(mid.segmentlist);
-    FREE(mid.segmentmarkerlist);
-    FREE(mid.edgelist);
-    FREE(mid.edgemarkerlist);
-    FREE(vorout.pointlist);
-    FREE(vorout.pointattributelist);
-    FREE(vorout.edgelist);
-    FREE(vorout.normlist);
+    free(in.pointlist);
+    free(in.pointattributelist);
+    free(in.pointmarkerlist);
+    free(in.regionlist);
+    free(mid.pointlist);
+    free(mid.pointattributelist);
+    free(mid.pointmarkerlist);
+    free(mid.triangleattributelist);
+    free(mid.neighborlist);
+    free(mid.segmentlist);
+    free(mid.segmentmarkerlist);
+    free(mid.edgelist);
+    free(mid.edgemarkerlist);
+    free(vorout.pointlist);
+    free(vorout.pointattributelist);
+    free(vorout.edgelist);
+    free(vorout.normlist);
 
     return mid.trianglelist;
 }
 
-// maybe it should be replaced by RNG - relative neigborhood graph, or by GG - gabriel graph
+// maybe it should be replaced by RNG - relative neighborhood graph, or by GG - gabriel graph
 int* 
 delaunay_tri (double *x, double *y, int n, int* nedges)
 {
@@ -691,8 +708,7 @@ delaunay_tri (double *x, double *y, int n, int* nedges)
     return out.edgelist;
 }
 
-v_data *delaunay_triangulation(double *x, double *y, int n)
-{
+static v_data *delaunay_triangulation(double *x, double *y, int n) {
     v_data *delaunay;
     int nedges;
     int *edges;
@@ -748,8 +764,7 @@ int* get_triangles (double *x, int n, int* tris)
     agerr(AGERR, "get_triangles: %s\n", err);
     return 0;
 }
-v_data *delaunay_triangulation(double *x, double *y, int n)
-{
+static v_data *delaunay_triangulation(double *x, double *y, int n) {
     agerr(AGERR, "delaunay_triangulation: %s\n", err);
     return 0;
 }
@@ -776,8 +791,7 @@ static void remove_edge(v_data * graph, int source, int dest)
     int i;
     for (i = 1; i < graph[source].nedges; i++) {
 	if (graph[source].edges[i] == dest) {
-	    graph[source].edges[i] =
-		graph[source].edges[--graph[source].nedges];
+	    graph[source].edges[i] = graph[source].edges[--graph[source].nedges];
 	    break;
 	}
     }
@@ -789,7 +803,6 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
     int i;
     double dist_ij, dist_ik, dist_jk, x_i, y_i, x_j, y_j;
     int j, k, neighbor_j, neighbor_k;
-    int removed;
 
     if (n == 2) {
 	int *edges = N_GNEW(4, int);
@@ -829,24 +842,17 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
 		}
 		x_j = x[neighbor_j];
 		y_j = y[neighbor_j];
-		dist_ij =
-		    (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
-		removed = FALSE;
+		dist_ij = (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
+		bool removed = false;
 		for (k = 0; k < n && !removed; k++) {
-		    dist_ik =
-			(x[k] - x_i) * (x[k] - x_i) + (y[k] -
-						       y_i) * (y[k] - y_i);
+		    dist_ik = (x[k] - x_i) * (x[k] - x_i) + (y[k] - y_i) * (y[k] - y_i);
 		    if (dist_ik < dist_ij) {
-			dist_jk =
-			    (x[k] - x_j) * (x[k] - x_j) + (y[k] -
-							   y_j) * (y[k] -
-								   y_j);
+			dist_jk = (x[k] - x_j) * (x[k] - x_j) + (y[k] - y_j) * (y[k] - y_j);
 			if (dist_jk < dist_ij) {
 			    // remove the edge beteween i and neighbor j
-			    delaunay[i].edges[j] =
-				delaunay[i].edges[--delaunay[i].nedges];
+			    delaunay[i].edges[j] = delaunay[i].edges[--delaunay[i].nedges];
 			    remove_edge(delaunay, neighbor_j, i);
-			    removed = TRUE;
+			    removed = true;
 			}
 		    }
 		}
@@ -864,26 +870,22 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
 		neighbor_j = delaunay[i].edges[j];
 		x_j = x[neighbor_j];
 		y_j = y[neighbor_j];
-		dist_ij =
-		    (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
+		dist_ij = (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
 		// now look at i'th neighbors to see whether there is a node in the "forbidden region"
 		// we will also go through neighbor_j's neighbors when we traverse the edge from its other side
-		removed = FALSE;
+		bool removed = false;
 		for (k = 1; k < delaunay[i].nedges && !removed; k++) {
 		    neighbor_k = delaunay[i].edges[k];
-		    dist_ik =
-			(x[neighbor_k] - x_i) * (x[neighbor_k] - x_i) +
+		    dist_ik = (x[neighbor_k] - x_i) * (x[neighbor_k] - x_i) +
 			(y[neighbor_k] - y_i) * (y[neighbor_k] - y_i);
 		    if (dist_ik < dist_ij) {
-			dist_jk =
-			    (x[neighbor_k] - x_j) * (x[neighbor_k] - x_j) +
+			dist_jk = (x[neighbor_k] - x_j) * (x[neighbor_k] - x_j) +
 			    (y[neighbor_k] - y_j) * (y[neighbor_k] - y_j);
 			if (dist_jk < dist_ij) {
 			    // remove the edge beteween i and neighbor j
-			    delaunay[i].edges[j] =
-				delaunay[i].edges[--delaunay[i].nedges];
+			    delaunay[i].edges[j] = delaunay[i].edges[--delaunay[i].nedges];
 			    remove_edge(delaunay, neighbor_j, i);
-			    removed = TRUE;
+			    removed = true;
 			}
 		    }
 		}
@@ -899,10 +901,8 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
 void freeGraph (v_data * graph)
 {
     if (graph != NULL) {
-	if (graph[0].edges != NULL)
-	    free(graph[0].edges);
-	if (graph[0].ewgts != NULL)
-	    free(graph[0].ewgts);
+	free(graph[0].edges);
+	free(graph[0].ewgts);
 	free(graph);
     }
 }
@@ -910,17 +910,13 @@ void freeGraph (v_data * graph)
 void freeGraphData(vtx_data * graph)
 {
     if (graph != NULL) {
-	if (graph[0].edges != NULL)
-	    free(graph[0].edges);
-	if (graph[0].ewgts != NULL)
-	    free(graph[0].ewgts);
+	free(graph[0].edges);
+	free(graph[0].ewgts);
 #ifdef USE_STYLES
-	if (graph[0].styles != NULL)
-	    free(graph[0].styles);
+	free(graph[0].styles);
 #endif
 #ifdef DIGCOLA
-	if (graph[0].edists != NULL)
-	    free(graph[0].edists);
+	free(graph[0].edists);
 #endif
 	free(graph);
     }
