@@ -1,6 +1,3 @@
-/* $Id$Revision: */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,17 +5,23 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 
-#include <partition.h>
-#include <trap.h>
-#include <memory.h>
+#include <ortho/partition.h>
+#include <ortho/trap.h>
+#include <common/memory.h>
 #include <math.h>
+#include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 
+#ifndef DEBUG
+  #define DEBUG 0
+#endif
+
 #define NPOINTS 4   /* only rectangles */
 #define TRSIZE(ss) (5*(ss)+1)
 
@@ -41,7 +44,7 @@
 #ifndef HAVE_SRAND48
 #define srand48 srand
 #endif
-#ifdef WIN32
+#ifdef _WIN32
 extern double drand48(void);
 #endif
 
@@ -126,7 +129,7 @@ store (segment_t* seg, int first, pointf* pts)
 	    seg[i].next = i+1;
 	    seg[i].prev = i-1;
 	}
-	seg[i].is_inserted = FALSE;
+	seg[i].is_inserted = false;
 	seg[seg[i].prev].v1 = seg[i].v0 = pts[j];
     }
     return (last+1);
@@ -163,23 +166,22 @@ generateRandomOrdering(int n, int* permute)
     }
 }
 
-/* Function returns TRUE if the trapezoid lies inside the polygon */
-static int 
+/* Function returns true if the trapezoid lies inside the polygon */
+static bool
 inside_polygon (trap_t *t, segment_t* seg)
 {
   int rseg = t->rseg;
 
   if (t->state == ST_INVALID)
-    return 0;
+    return false;
 
-  if ((t->lseg <= 0) || (t->rseg <= 0))
-    return 0;
+  if (t->lseg <= 0 || t->rseg <= 0)
+    return false;
   
-  if (((t->u0 <= 0) && (t->u1 <= 0)) || 
-      ((t->d0 <= 0) && (t->d1 <= 0))) /* triangle */
-    return (_greater_than(&seg[rseg].v1, &seg[rseg].v0));
+  if ((t->u0 <= 0 && t->u1 <= 0) || (t->d0 <= 0 && t->d1 <= 0)) /* triangle */
+    return _greater_than(&seg[rseg].v1, &seg[rseg].v0);
   
-  return 0;
+  return false;
 }
 
 static double
@@ -196,16 +198,16 @@ get_angle (pointf *vp0, pointf *vpnext, pointf *vp1)
   if (CROSS_SINE(v0, v1) >= 0)	/* sine is positive */
     return DOT(v0, v1)/LENGTH(v0)/LENGTH(v1);
   else
-    return (-1.0 * DOT(v0, v1)/LENGTH(v0)/LENGTH(v1) - 2);
+    return -1.0 * DOT(v0, v1)/LENGTH(v0)/LENGTH(v1) - 2;
 }
 
 /* (v0, v1) is the new diagonal to be added to the polygon. Find which */
 /* chain to use and return the positions of v0 and v1 in p and q */ 
-static int
+static void
 get_vertex_positions (int v0, int v1, int *ip, int *iq)
 {
   vertexchain_t *vp0, *vp1;
-  register int i;
+  int i;
   double angle, temp;
   int tp = 0, tq = 0;
 
@@ -247,8 +249,6 @@ get_vertex_positions (int v0, int v1, int *ip, int *iq)
     }
 
   *iq = tq;
-
-  return 0;
 }
 
 /* v0 and v1 are specified in anti-clockwise order with respect to 
@@ -303,9 +303,8 @@ make_new_monotone_poly (int mcur, int v0, int v1)
   vp0->nextfree++;
   vp1->nextfree++;
 
-#ifdef DEBUG
-  fprintf(stderr, "make_poly: mcur = %d, (v0, v1) = (%d, %d)\n", 
-	  mcur, v0, v1);
+#if DEBUG > 0
+  fprintf(stderr, "make_poly: mcur = %d, (v0, v1) = (%d, %d)\n", mcur, v0, v1);
   fprintf(stderr, "next posns = (p, q) = (%d, %d)\n", p, q);
 #endif
 
@@ -319,20 +318,19 @@ static int
 traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t* tr,
     int mcur, int trnum, int from, int flip, int dir)
 {
-  trap_t *t = &tr[trnum];
+  trap_t *t;
   int mnew;
   int v0, v1;
-  int retval;
-  int do_switch = FALSE;
 
-  if ((trnum <= 0) || visited[trnum])
+  if (trnum <= 0 || visited[trnum])
     return size;
 
+  t = &tr[trnum];
+
   visited[trnum] = TRUE;
   
-  if ((t->hi.y > t->lo.y) &&
-      (seg[t->lseg].v0.x == seg[t->lseg].v1.x) &&
-      (seg[t->rseg].v0.x == seg[t->rseg].v1.x)) {
+  if (t->hi.y > t->lo.y && seg[t->lseg].v0.x == seg[t->lseg].v1.x &&
+      seg[t->rseg].v0.x == seg[t->rseg].v1.x) {
       if (flip) {
           decomp[size].LL.x = t->lo.y;
           decomp[size].LL.y = -seg[t->rseg].v0.x;
@@ -357,15 +355,14 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 
   /* special cases for triangles with cusps at the opposite ends. */
   /* take care of this first */
-  if ((t->u0 <= 0) && (t->u1 <= 0))
+  if (t->u0 <= 0 && t->u1 <= 0)
     {
-      if ((t->d0 > 0) && (t->d1 > 0)) /* downward opening triangle */
+      if (t->d0 > 0 && t->d1 > 0) /* downward opening triangle */
 	{
 	  v0 = tr[t->d1].lseg;
 	  v1 = t->lseg;
 	  if (from == t->d1)
 	    {
-	      do_switch = TRUE;
 	      mnew = make_new_monotone_poly(mcur, v1, v0);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d1, trnum, flip, TR_FROM_UP);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mnew, t->d0, trnum, flip, TR_FROM_UP);	    
@@ -379,7 +376,7 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	}
       else
 	{
-	  retval = SP_NOSPLIT;	/* Just traverse all neighbours */
+	  /* Just traverse all neighbours */
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d0, trnum, flip, TR_FROM_UP);
@@ -387,15 +384,14 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	}
     }
   
-  else if ((t->d0 <= 0) && (t->d1 <= 0))
+  else if (t->d0 <= 0 && t->d1 <= 0)
     {
-      if ((t->u0 > 0) && (t->u1 > 0)) /* upward opening triangle */
+      if (t->u0 > 0 && t->u1 > 0) /* upward opening triangle */
 	{
 	  v0 = t->rseg;
 	  v1 = tr[t->u0].rseg;
 	  if (from == t->u1)
 	    {
-	      do_switch = TRUE;
 	      mnew = make_new_monotone_poly(mcur, v1, v0);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mnew, t->u0, trnum, flip, TR_FROM_DN);	    
@@ -409,7 +405,7 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	}
       else
 	{
-	  retval = SP_NOSPLIT;	/* Just traverse all neighbours */
+	  /* Just traverse all neighbours */
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 	  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d0, trnum, flip, TR_FROM_UP);
@@ -417,17 +413,15 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	}
     }
   
-  else if ((t->u0 > 0) && (t->u1 > 0)) 
+  else if (t->u0 > 0 && t->u1 > 0)
     {
-      if ((t->d0 > 0) && (t->d1 > 0)) /* downward + upward cusps */
+      if (t->d0 > 0 && t->d1 > 0) /* downward + upward cusps */
 	{
 	  v0 = tr[t->d1].lseg;
 	  v1 = tr[t->u0].rseg;
-	  retval = SP_2UP_2DN;
-	  if (((dir == TR_FROM_DN) && (t->d1 == from)) ||
-	      ((dir == TR_FROM_UP) && (t->u1 == from)))
+	  if ((dir == TR_FROM_DN && t->d1 == from) ||
+	      (dir == TR_FROM_UP && t->u1 == from))
 	    {
-	      do_switch = TRUE;
 	      mnew = make_new_monotone_poly(mcur, v1, v0);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d1, trnum, flip, TR_FROM_UP);
@@ -450,10 +444,8 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	      v0 = tr[t->u0].rseg;
 	      v1 = seg[t->lseg].next;
 
-	      retval = SP_2UP_LEFT;
-	      if ((dir == TR_FROM_UP) && (t->u0 == from))
+	      if (dir == TR_FROM_UP && t->u0 == from)
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mnew, t->d0, trnum, flip, TR_FROM_UP);
@@ -473,10 +465,8 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	    {
 	      v0 = t->rseg;
 	      v1 = tr[t->u0].rseg;	
-	      retval = SP_2UP_RIGHT;
-	      if ((dir == TR_FROM_UP) && (t->u1 == from))
+	      if (dir == TR_FROM_UP && t->u1 == from)
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mnew, t->d1, trnum, flip, TR_FROM_UP);
@@ -494,18 +484,16 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	    }
 	}
     }
-  else if ((t->u0 > 0) || (t->u1 > 0)) /* no downward cusp */
+  else if (t->u0 > 0 || t->u1 > 0) /* no downward cusp */
     {
-      if ((t->d0 > 0) && (t->d1 > 0)) /* only upward cusp */
+      if (t->d0 > 0 && t->d1 > 0) /* only upward cusp */
 	{
 	  if (_equal_to(&t->hi, &seg[t->lseg].v0))
 	    {
 	      v0 = tr[t->d1].lseg;
 	      v1 = t->lseg;
-	      retval = SP_2DN_LEFT;
-	      if (!((dir == TR_FROM_DN) && (t->d0 == from)))
+	      if (!(dir == TR_FROM_DN && t->d0 == from))
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d1, trnum, flip, TR_FROM_UP);
@@ -526,10 +514,8 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	      v0 = tr[t->d1].lseg;
 	      v1 = seg[t->rseg].next;
 
-	      retval = SP_2DN_RIGHT;	    
-	      if ((dir == TR_FROM_DN) && (t->d1 == from))
+	      if (dir == TR_FROM_DN && t->d1 == from)
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d1, trnum, flip, TR_FROM_UP);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mnew, t->u1, trnum, flip, TR_FROM_DN);
@@ -553,10 +539,8 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	    {
 	      v0 = t->rseg;
 	      v1 = t->lseg;
-	      retval = SP_SIMPLE_LRDN;
 	      if (dir == TR_FROM_UP)
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
@@ -578,10 +562,8 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	      v0 = seg[t->rseg].next;
 	      v1 = seg[t->lseg].next;
 
-	      retval = SP_SIMPLE_LRUP;
 	      if (dir == TR_FROM_UP)
 		{
-		  do_switch = TRUE;
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 		  size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
@@ -599,7 +581,6 @@ traverse_polygon (int* visited, boxf* decomp, int size, segment_t* seg, trap_t*
 	    }
 	  else			/* no split possible */
 	    {
-	      retval = SP_NOSPLIT;
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u0, trnum, flip, TR_FROM_DN);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->d0, trnum, flip, TR_FROM_UP);
 	      size = traverse_polygon (visited, decomp, size, seg, tr, mcur, t->u1, trnum, flip, TR_FROM_DN);
@@ -665,24 +646,18 @@ monotonate_trapezoids(int nsegs, segment_t*seg, trap_t* tr,
   return size;
 }
 
-static int 
+static bool
 rectIntersect (boxf *d, const boxf *r0, const boxf *r1)
 {
-    double t;
-
-    t = (r0->LL.x > r1->LL.x) ? r0->LL.x : r1->LL.x;
-    d->UR.x = (r0->UR.x < r1->UR.x) ? r0->UR.x : r1->UR.x;
+    double t = fmax(r0->LL.x, r1->LL.x);
+    d->UR.x = fmin(r0->UR.x, r1->UR.x);
     d->LL.x = t;
     
-    t = (r0->LL.y > r1->LL.y) ? r0->LL.y : r1->LL.y;
-    d->UR.y = (r0->UR.y < r1->UR.y) ? r0->UR.y : r1->UR.y;
+    t = fmax(r0->LL.y, r1->LL.y);
+    d->UR.y = fmin(r0->UR.y, r1->UR.y);
     d->LL.y = t;
 
-    if ((d->LL.x >= d->UR.x) ||
-        (d->LL.y >= d->UR.y))
-    return 0;
-
-    return 1;
+    return !(d->LL.x >= d->UR.x || d->LL.y >= d->UR.y);
 }
 
 #if DEBUG > 1
@@ -709,7 +684,7 @@ dumpSegs (segment_t* sg, int n)
       sg++;
       fprintf (stderr, "%d : (%f,%f) (%f,%f) %d %d %d %d %d\n", i,
          sg->v0.x, sg->v0.y, sg->v1.x, sg->v1.y,
-         sg->is_inserted, sg->root0,  sg->root1, sg->next, sg->prev);
+         (int)sg->is_inserted, sg->root0,  sg->root1, sg->next, sg->prev);
     }
     fprintf (stderr, "====\n");
 }
@@ -730,26 +705,32 @@ partition (cell* cells, int ncells, int* nrects, boxf bb)
     boxf* vert_decomp = N_NEW(ntraps, boxf);
     int nt;
 
-    /* fprintf (stderr, "cells = %d segs = %d traps = %d\n", ncells, nsegs, ntraps);  */
+    if (DEBUG) {
+	fprintf (stderr, "cells = %d segs = %d traps = %d\n", ncells, nsegs, ntraps);
+    }
     genSegments (cells, ncells, bb, segs, 0);
-#if 0
-fprintf (stderr, "%d\n\n", ncells+1);
-for (i = 1; i<= nsegs; i++) {
-  if (i%4 == 1) fprintf(stderr, "4\n");
-  fprintf (stderr, "%f %f\n", segs[i].v0.x, segs[i].v0.y);
-  if (i%4 == 0) fprintf(stderr, "\n");
-}
-#endif
+    if (DEBUG) {
+	fprintf (stderr, "%d\n\n", ncells+1);
+	for (i = 1; i<= nsegs; i++) {
+	    if (i%4 == 1) fprintf(stderr, "4\n");
+	    fprintf (stderr, "%f %f\n", segs[i].v0.x, segs[i].v0.y);
+	    if (i%4 == 0) fprintf(stderr, "\n");
+	}
+    }
     srand48(173);
     generateRandomOrdering (nsegs, permute);
     nt = construct_trapezoids(nsegs, segs, permute, ntraps, trs);
-    /* fprintf (stderr, "hor traps = %d\n", nt); */
+    if (DEBUG) {
+	fprintf (stderr, "hor traps = %d\n", nt);
+    }
     hd_size = monotonate_trapezoids (nsegs, segs, trs, 0, hor_decomp);
 
     genSegments (cells, ncells, bb, segs, 1);
     generateRandomOrdering (nsegs, permute);
     nt = construct_trapezoids(nsegs, segs, permute, ntraps, trs);
-    /* fprintf (stderr, "ver traps = %d\n", nt); */
+    if (DEBUG) {
+	fprintf (stderr, "ver traps = %d\n", nt);
+    }
     vd_size = monotonate_trapezoids (nsegs, segs, trs, 1, vert_decomp);
 
     rs = N_NEW (hd_size*vd_size, boxf);
